/**
 * BIBAUTIL.H -- a set of operations on biba text
 *
 */

#ifndef _BIBAUTIL_H
#define _BIBAUTIL_H

#include <sys/types.h>

#define BIBA_LABLE_TEXT_MAX_LEN 4096 /* The extreme situation is g30.t0-t0,t1-t1,...,t223-t223~g30.t0-t0,t1-t1,...,t223-t223 , which length is 4047 */

/* some magics */
#define BIBAFS_MAGIC           0xB1BA0622
#define BIBAFS_LABEL_IN_MAGIC  0xB1BA1111
#define BIBAFS_LABEL_OUT_MAGIC 0xB1BA0000

/* some operating codes */
#define BOP_GET_FILE_BIBA      0x00000001
#define BOP_SET_FILE_BIBA      0x00000002
#define BOP_GET_PROC_BIBA      0x00000003
#define BOP_SET_PROC_BIBA      0x00000004
#define BOP_GET_USER_BIBA      0x00000005
#define BOP_SET_USER_BIBA      0x00000006
#define BOP_GET_USER_RANGE     0x00000007
#define BOP_GET_FS_BIBA        0x00000008
#define BOP_SET_FS_BIBA        0x00000009

/* some macro tools */
#define NORMAL_BIBA_STAT(x) ((BIBA_ENABLE == (x) || BIBA_DISABLE == (x)) ? 1 : 0)

/* struct for bibafs/label operations */
struct bibafs_label_args {
	long magic;          /* indicate read() to read or write */
	int  op;             /* indicate what operation on what type of object for read() */
	int  len;            /* if value stores biba context, it indicate the length of value.str, otherwise no used */ 
	union {
		pid_t pid;
		uid_t uid;
		char  str[0];
	} value;
};

/* enum all of biba status */
enum biba_stat_flag {
	BIBA_UNKNOWN = -2, /* unknow status, used for some failed gets operations */
	BIBA_ERROR = -1, /* unnormal status of biba */
	BIBA_ENABLE,     /* enable status of biba */
	BIBA_DISABLE     /* disable status of biba */
};

/* get file biba label context */
#ifdef SHARED
int (*bop_get_file_biba)(const char *pathname, char *label);
int (*bop_nget_file_biba)(const char *pathname, char *label, size_t len);
#else
int bop_get_file_biba(const char *pathname, char *label);
int bop_nget_file_biba(const char *pathname, char *label, size_t len);
#endif

/* get filesystem biba label context */
#ifdef SHARED
int (*bop_get_fs_biba)(const char* fsname, char *label);
int (*bop_nget_fs_biba)(const char* fsname, char *label size_t len);
#else
int bop_get_fs_biba(const char *fsname, char *label);
int bop_nget_fs_biba(const char *fsname, char *label, size_t len);
#endif

/* get user biba label context */
#ifdef SHARED
int (*bop_get_user_biba)(uid_t uid, char *label);
int (*bop_nget_user_biba)(uid_t uid, char *label, size_t len);
#else
int bop_get_user_biba(uid_t uid, char *label);
int bop_nget_user_biba(uid_t uid, char *label, size_t len);
#endif

/* get user biba label range context */
#ifdef SHARED
int (*bop_get_user_range)(uid_t uid, char *label);
int (*bop_nget_user_range)(uid_t uid, char *label, size_t len);
#else
int bop_get_user_range(uid_t uid, char *label);
int bop_nget_user_range(uid_t uid, char *label, size_t len);
#endif

/* get process biba label */
#ifdef SHARED
int (*bop_get_proc_biba)(pid_t pid, char *label);
int (*bop_nget_proc_biba)(pid_t pid, char *label, size_t len);
#else
int bop_get_proc_biba(pid_t pid, char *label);
int bop_nget_proc_biba(pid_t pid, char *label, size_t len);
#endif

/* set file biba label */
#ifdef SHARED
int (*bop_set_file_biba)(const char* pathname, const char* label);
#else
int bop_set_file_biba(const char* pathname, const char* label);
#endif

/* set file system biba label */
#ifdef SHARED
int (*bop_set_fs_biba)(const char* fsname, const char* label);
#else
int bop_set_fs_biba(const char*fsname, const char* label);
#endif

/* set user biba label */
#ifdef SHARED
int (*bop_set_user_biba)(const char* label);
#else
int bop_set_user_biba(const char* label);
#endif

/* set proc biba label */
#ifdef SHARED
int (*bop_set_proc_biba)(const char* label);
#else
int bop_set_proc_biba(const char* label);
#endif

/* pos where biba fs is mounted */
#ifdef SHARED
int (*bop_pos_bibafs_mntp)(char* path, size_t len);
#else
int bop_pos_bibafs_mntp(char *path, size_t len);
#endif

/* open files about biba on fs */
#ifdef SHARED
int (*bop_open_file)(const char* path, const char* file, int flags);
#else
int bop_open_file(const char* path, const char* file, int flags);
#endif

/* open biba label on fs */
#ifdef SHARED
int (*bop_open_label_file)(const char* file);
#else
int bop_open_label_file(const char* file);
#endif

/* load biba policy from file */
#ifdef SHARED
	int (*bop_load_biba_policy)(const char* pathname, const char* pol_file);
	int (*biba_init_load_policy)(enum biba_stat_flag *status);
#else
	int bop_load_biba_policy(const char* pathname, const char* pol_file);
	int biba_init_load_policy(enum biba_stat_flag *status);
#endif

/* enable biba */
#ifdef SHARED
	int (*bop_set_biba_stat)(enum biba_stat_flag, const char* en_file);
#else
	int bop_set_biba_stat(enum biba_stat_flag, const char* en_file);
#endif

/* get biba enable status */	
#ifdef SHARED
	enum biba_stat_flag (*bop_get_biba_stat)(const char* en_file);
#else
	enum biba_stat_flag bop_get_biba_stat(const char* en_file);
#endif


/* user judge */
#ifdef SHARED
	void bop_check_user(const char* oldpolicy, const char * newpolicy);
#else
	void bop_check_user(const char* oldpolicy, const char * newpolicy);
#endif
/* valid label */
#ifdef SHARED
	int (*valid_label)(const char* label);
#else
	int valid_label(const char* label);
#endif

#endif  /* bibautil.h */
